@@ -32,12 +32,10 @@
 #include <time.h>
 
 #include <cairo.h>
-#include <cairo-xlib.h>
 
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gio/gio.h>
 
@@ -83,6 +81,9 @@ typedef struct {
 	GtkRequisition     requisition;
 	GdkRectangle       prev_allocation;
 	cairo_surface_t   *surface;
+	gint               surface_width;
+	gint               surface_height;
+
 	guint              timeout;
 	int                current_frame;
 	gboolean           in_applet;
@@ -400,8 +401,6 @@ static void display_preferences_dialog(GtkAction* action, FishApplet* fish)
 	g_object_add_weak_pointer (G_OBJECT (fish->preferences_dialog),
 				   (void**) &fish->preferences_dialog);
 
-	gtk_window_set_wmclass (GTK_WINDOW (fish->preferences_dialog),
-				"fish", "Fish");
 	gtk_window_set_icon_name (GTK_WINDOW (fish->preferences_dialog),
 				  FISH_ICON);
 	gtk_dialog_set_default_response (
@@ -548,9 +547,6 @@ static void display_about_dialog(GtkAction* action, FishApplet* fish)
 
 	char* authors[3];
 	char* descr;
-	char copyright[] = \
-		"Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.\n"
-		"Copyright \xc2\xa9 2012-2018 MATE developers";
 
 	authors[0] = g_strdup_printf(author_format, fish->name);
 	authors[1] = _("(with minor help from George)");
@@ -560,9 +556,12 @@ static void display_about_dialog(GtkAction* action, FishApplet* fish)
 
 	gtk_show_about_dialog(NULL,
 		"program-name", _("Fish"),
+		"title", _("About Fish"),
 		"authors", authors,
 		"comments", descr,
-		"copyright", copyright,
+		"copyright", _("Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.\n"
+		               "Copyright \xc2\xa9 2002-2005 Vincent Untz\n"
+		               "Copyright \xc2\xa9 2012-2019 MATE developers"),
 		"documenters", documenters,
 		"logo-icon-name", FISH_ICON,
 		"translator-credits", _("translator-credits"),
@@ -840,9 +839,8 @@ static void display_fortune_dialog(FishApplet* fish)
 	if (!fish->fortune_dialog) {
 		GtkWidget *scrolled;
 		GtkWidget *vbox;
-		GdkScreen *screen;
-		int        screen_width;
-		int        screen_height;
+		GdkMonitor *monitor;
+		GdkRectangle monitor_geom;
 
 		fish->fortune_dialog = gtk_dialog_new ();
 		gtk_window_set_title (GTK_WINDOW (fish->fortune_dialog), "");
@@ -866,16 +864,12 @@ static void display_fortune_dialog(FishApplet* fish)
 		g_signal_connect (fish->fortune_dialog, "response",
 				  G_CALLBACK (handle_fortune_response), fish);
 
-		gtk_window_set_wmclass (GTK_WINDOW (fish->fortune_dialog), "fish", "Fish");
-
-		screen = gtk_widget_get_screen (GTK_WIDGET (fish));
-
-		screen_width  = WidthOfScreen (gdk_x11_screen_get_xscreen (screen));
-		screen_height = HeightOfScreen (gdk_x11_screen_get_xscreen (screen));
-
+		monitor = gdk_display_get_monitor_at_window (gtk_widget_get_display (GTK_WIDGET (fish)),
+							     gtk_widget_get_window (GTK_WIDGET (fish)));
+		gdk_monitor_get_geometry(monitor, &monitor_geom);
 		gtk_window_set_default_size (GTK_WINDOW (fish->fortune_dialog),
-					     MIN (600, screen_width  * 0.9),
-					     MIN (350, screen_height * 0.9));
+					     MIN (600, monitor_geom.width  * 0.9),
+					     MIN (350, monitor_geom.height * 0.9));
 
 		fish->fortune_view = gtk_text_view_new ();
 		gtk_text_view_set_editable (GTK_TEXT_VIEW (fish->fortune_view), FALSE);
@@ -1423,6 +1417,8 @@ static void update_pixmap(FishApplet* fish)
 	fish->surface = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
 													   CAIRO_CONTENT_COLOR_ALPHA,
 													   width, height);
+	fish->surface_width = width;
+	fish->surface_height = height;
 
 	gtk_widget_queue_resize (widget);
 
@@ -1484,8 +1480,8 @@ static gboolean fish_applet_draw(GtkWidget* widget, cairo_t *cr, FishApplet* fis
 
 	g_assert (fish->n_frames > 0);
 
-	width = cairo_xlib_surface_get_width (fish->surface);
-	height = cairo_xlib_surface_get_height (fish->surface);
+	width = fish->surface_width;
+	height = fish->surface_height;
 	src_x = 0;
 	src_y = 0;
 
@@ -1528,6 +1524,8 @@ static void fish_applet_unrealize(GtkWidget* widget, FishApplet* fish)
 	if (fish->surface)
 		cairo_surface_destroy (fish->surface);
 	fish->surface = NULL;
+	fish->surface_width = 0;
+	fish->surface_height = 0;
 }
 
 static void fish_applet_change_orient(MatePanelApplet* applet, MatePanelAppletOrient orientation)
@@ -1554,7 +1552,6 @@ static void change_water(FishApplet* fish)
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  _("Look at today's date!"));
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), FISH_ICON);
-	gtk_window_set_wmclass (GTK_WINDOW (dialog), "fish", "Fish");
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
 
@@ -1814,6 +1811,8 @@ static void fish_applet_dispose (GObject *object)
 	if (fish->surface)
 		cairo_surface_destroy (fish->surface);
 	fish->surface = NULL;
+	fish->surface_width = 0;
+	fish->surface_height = 0;
 
 	if (fish->pixbuf)
 		g_object_unref (fish->pixbuf);
