@@ -22,15 +22,22 @@
  */
 
 #include <config.h>
+
+#ifndef HAVE_X11
+#error file should only be built when HAVE_X11 is enabled
+#endif
+
 #include <string.h>
 
 #include <mate-panel-applet.h>
+#include <mate-panel-applet-gsettings.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 
 #include "main.h"
-#include "na-box.h"
+#include "na-grid.h"
 
 #ifdef PROVIDE_WATCHER_SERVICE
 # include "libstatus-notifier-watcher/gf-status-notifier-watcher.h"
@@ -38,9 +45,21 @@
 
 #define NOTIFICATION_AREA_ICON "mate-panel-notification-area"
 
+typedef struct
+{
+  GtkWidget *preferences_dialog;
+  GtkWidget *min_icon_size_spin;
+} NAPreferencesDialog;
+
 struct _NaTrayAppletPrivate
 {
-  GtkWidget *box;
+  GtkWidget               *grid;
+
+  NAPreferencesDialog     *dialog;
+  GtkBuilder              *builder;
+
+  GSettings               *settings;
+  gint                     min_icon_size;
 
 #ifdef PROVIDE_WATCHER_SERVICE
   GfStatusNotifierWatcher *sn_watcher;
@@ -104,6 +123,99 @@ get_gtk_orientation_from_applet_orient (MatePanelAppletOrient orient)
   return GTK_ORIENTATION_HORIZONTAL;
 }
 
+static void
+gsettings_changed_min_icon_size (GSettings    *settings,
+                                 gchar        *key,
+                                 NaTrayApplet *applet)
+{
+  applet->priv->min_icon_size = g_settings_get_int (settings, key);
+
+  if (applet->priv->dialog)
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON (applet->priv->dialog->min_icon_size_spin),
+                               applet->priv->min_icon_size);
+
+  na_grid_set_min_icon_size (NA_GRID (applet->priv->grid), applet->priv->min_icon_size);
+}
+
+static void
+setup_gsettings (NaTrayApplet *applet)
+{
+  applet->priv->settings = mate_panel_applet_settings_new (MATE_PANEL_APPLET (applet), NA_TRAY_SCHEMA);
+  g_signal_connect (applet->priv->settings, "changed::" KEY_MIN_ICON_SIZE, G_CALLBACK (gsettings_changed_min_icon_size), applet);
+}
+
+static void
+na_preferences_dialog_min_icon_size_changed (NaTrayApplet  *applet,
+                                             GtkSpinButton *spin_button)
+{
+  applet->priv->min_icon_size = gtk_spin_button_get_value_as_int (spin_button);
+  g_settings_set_int (applet->priv->settings, KEY_MIN_ICON_SIZE, applet->priv->min_icon_size);
+}
+
+static gboolean
+na_preferences_dialog_hide_event (GtkWidget    *widget,
+                                  GdkEvent     *event,
+                                  NaTrayApplet *applet)
+{
+  gtk_widget_hide (applet->priv->dialog->preferences_dialog);
+  return TRUE;
+}
+
+static void
+na_preferences_dialog_response (NaTrayApplet *applet,
+                                int           response,
+                                GtkWidget    *preferences_dialog)
+{
+  switch (response)
+    {
+    case GTK_RESPONSE_CLOSE:
+      gtk_widget_hide (preferences_dialog);
+      break;
+    default:
+      break;
+    }
+}
+
+static void
+ensure_prefs_window_is_created (NaTrayApplet *applet)
+{  
+  if (applet->priv->dialog)
+    return;
+
+  applet->priv->dialog = g_new0 (NAPreferencesDialog, 1);
+  
+  applet->priv->dialog->preferences_dialog = GTK_WIDGET (gtk_builder_get_object (applet->priv->builder, "notification_area_preferences_dialog"));
+
+  gtk_window_set_icon_name (GTK_WINDOW (applet->priv->dialog->preferences_dialog), NOTIFICATION_AREA_ICON);
+  
+  applet->priv->dialog->min_icon_size_spin = GTK_WIDGET (gtk_builder_get_object (applet->priv->builder, "min_icon_size_spin"));
+  g_return_if_fail (applet->priv->dialog->min_icon_size_spin != NULL);
+  
+  gtk_spin_button_set_range (GTK_SPIN_BUTTON (applet->priv->dialog->min_icon_size_spin), 7, 100);
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (applet->priv->dialog->min_icon_size_spin), applet->priv->min_icon_size);
+
+  g_signal_connect_swapped (applet->priv->dialog->min_icon_size_spin, "value_changed",
+                            G_CALLBACK (na_preferences_dialog_min_icon_size_changed),
+                            applet);
+
+  g_signal_connect_swapped (applet->priv->dialog->preferences_dialog, "response",
+                            G_CALLBACK (na_preferences_dialog_response), applet);
+
+  g_signal_connect (G_OBJECT (applet->priv->dialog->preferences_dialog), "delete_event",
+                    G_CALLBACK (na_preferences_dialog_hide_event), applet);
+}
+
+static void
+properties_dialog (GtkAction    *action,
+                   NaTrayApplet *applet)
+{
+  ensure_prefs_window_is_created (applet);
+
+  gtk_window_set_screen (GTK_WINDOW (applet->priv->dialog->preferences_dialog),
+                         gtk_widget_get_screen (GTK_WIDGET (applet)));
+  gtk_window_present (GTK_WINDOW (applet->priv->dialog->preferences_dialog));
+}
+
 static void help_cb(GtkAction* action, NaTrayApplet* applet)
 {
 	GError* error = NULL;
@@ -151,6 +263,7 @@ static void about_cb(GtkAction* action, NaTrayApplet* applet)
 		"Vincent Untz <vuntz@gnome.org>",
 		"Alberts Muktupāvels",
 		"Colomban Wendling <cwendling@hypra.fr>",
+		"Fabien Broquard <braikar@gmail.com>",
 		NULL
 	};
 
@@ -159,17 +272,15 @@ static void about_cb(GtkAction* action, NaTrayApplet* applet)
 		NULL
 	};
 
-	const char copyright[] = \
-		"Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
-		"Copyright \xc2\xa9 2003-2006 Vincent Untz\n"
-		"Copyright \xc2\xa9 2011 Perberos\n"
-		"Copyright \xc2\xa9 2012-2018 MATE developers";
-
 	gtk_show_about_dialog(NULL,
 		"program-name", _("Notification Area"),
+		"title", _("About Notification Area"),
 		"authors", authors,
 		//"comments", _(comments),
-		"copyright", copyright,
+		"copyright", _("Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
+		               "Copyright \xc2\xa9 2003-2006 Vincent Untz\n"
+		               "Copyright \xc2\xa9 2011 Perberos\n"
+		               "Copyright \xc2\xa9 2012-2019 MATE developers"),
 		"documenters", documenters,
 		"logo-icon-name", NOTIFICATION_AREA_ICON,
 		"translator-credits", _("translator-credits"),
@@ -178,6 +289,9 @@ static void about_cb(GtkAction* action, NaTrayApplet* applet)
 }
 
 static const GtkActionEntry menu_actions [] = {
+	{ "SystemTrayPreferences", "document-properties", N_("_Preferences"),
+	  NULL, NULL,
+	  G_CALLBACK (properties_dialog) },
 	{ "SystemTrayHelp", "help-browser", N_("_Help"),
 	  NULL, NULL,
 	  G_CALLBACK (help_cb) },
@@ -203,15 +317,27 @@ na_tray_applet_realize (GtkWidget *widget)
                                               NA_RESOURCE_PATH "notification-area-menu.xml",
                                               action_group);
   g_object_unref(action_group);
+
+  setup_gsettings (applet);
+
+  // load min icon size
+  gsettings_changed_min_icon_size (applet->priv->settings, KEY_MIN_ICON_SIZE, applet);
+
+  applet->priv->builder = gtk_builder_new ();
+  gtk_builder_set_translation_domain (applet->priv->builder, GETTEXT_PACKAGE);
+  gtk_builder_add_from_resource (applet->priv->builder, NA_RESOURCE_PATH "notification-area-preferences-dialog.ui", NULL);
 }
 
 static void
 na_tray_applet_dispose (GObject *object)
 {
+  g_clear_object (&NA_TRAY_APPLET (object)->priv->settings);
 #ifdef PROVIDE_WATCHER_SERVICE
   g_clear_object (&NA_TRAY_APPLET (object)->priv->sn_watcher);
 #endif
 
+  g_clear_object (&NA_TRAY_APPLET (object)->priv->builder);
+
   G_OBJECT_CLASS (na_tray_applet_parent_class)->dispose (object);
 }
 
@@ -225,14 +351,14 @@ na_tray_applet_style_updated (GtkWidget *widget)
   if (parent_class_style_updated)
     parent_class_style_updated (widget);
 
-  if (!applet->priv->box)
+  if (!applet->priv->grid)
     return;
 
   gtk_widget_style_get (widget,
                         "icon-padding", &padding,
                         "icon-size", &icon_size,
                         NULL);
-  g_object_set (applet->priv->box,
+  g_object_set (applet->priv->grid,
                 "icon-padding", padding,
                 "icon-size", icon_size,
                 NULL);
@@ -247,8 +373,8 @@ na_tray_applet_change_background(MatePanelApplet* panel_applet, MatePanelAppletB
     parent_class_change_background (panel_applet, type, color, pattern);
   }
 
-  if (applet->priv->box)
-    na_box_force_redraw (NA_BOX (applet->priv->box));
+  if (applet->priv->grid)
+    na_grid_force_redraw (NA_GRID (applet->priv->grid));
 }
 
 static void
@@ -260,10 +386,10 @@ na_tray_applet_change_orient (MatePanelApplet       *panel_applet,
   if (parent_class_change_orient)
     parent_class_change_orient (panel_applet, orient);
 
-  if (!applet->priv->box)
+  if (!applet->priv->grid)
     return;
 
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->priv->box),
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->priv->grid),
                                   get_gtk_orientation_from_applet_orient (orient));
 }
 
@@ -286,10 +412,10 @@ na_tray_applet_focus (GtkWidget        *widget,
 {
   NaTrayApplet *applet = NA_TRAY_APPLET (widget);
 
-  /* We let the box handle the focus movement because we behave more like a
+  /* We let the grid handle the focus movement because we behave more like a
    * container than a single applet.  But if focus didn't move, we let the
    * applet do its thing. */
-  if (gtk_widget_child_focus (applet->priv->box, direction))
+  if (gtk_widget_child_focus (applet->priv->grid, direction))
     return TRUE;
 
   return GTK_WIDGET_CLASS (na_tray_applet_parent_class)->focus (widget, direction);
@@ -353,10 +479,10 @@ na_tray_applet_init (NaTrayApplet *applet)
 #endif
 
   orient = mate_panel_applet_get_orient (MATE_PANEL_APPLET (applet));
-  applet->priv->box = na_box_new (get_gtk_orientation_from_applet_orient (orient));
+  applet->priv->grid = na_grid_new (get_gtk_orientation_from_applet_orient (orient));
 
-  gtk_container_add (GTK_CONTAINER (applet), GTK_WIDGET (applet->priv->box));
-  gtk_widget_show (GTK_WIDGET (applet->priv->box));
+  gtk_container_add (GTK_CONTAINER (applet), GTK_WIDGET (applet->priv->grid));
+  gtk_widget_show (GTK_WIDGET (applet->priv->grid));
 
   atko = gtk_widget_get_accessible (GTK_WIDGET (applet));
   atk_object_set_name (atko, _("Panel Notification Area"));
@@ -374,6 +500,11 @@ applet_factory (MatePanelApplet *applet,
         strcmp (iid, "SystemTrayApplet") == 0))
     return FALSE;
 
+  if (!GDK_IS_X11_DISPLAY (gtk_widget_get_display (GTK_WIDGET (applet)))) {
+    g_warning ("Notification area only works on X");
+    return FALSE;
+  }
+
 #ifndef NOTIFICATION_AREA_INPROCESS
   gtk_window_set_default_icon_name (NOTIFICATION_AREA_ICON);
 #endif
