@@ -20,6 +20,11 @@
  */
 
 #include <config.h>
+
+#ifndef HAVE_X11
+#error file should only be built when HAVE_X11 is enabled
+#endif
+
 #include <string.h>
 
 #include "na-tray-child.h"
@@ -400,6 +405,7 @@ na_tray_child_new (GdkScreen *screen,
 {
   XWindowAttributes window_attributes;
   Display *xdisplay;
+  GdkDisplay *display;
   NaTrayChild *child;
   GdkVisual *visual;
   gboolean visual_has_alpha;
@@ -415,10 +421,15 @@ na_tray_child_new (GdkScreen *screen,
    * the socket in the same visual.
    */
 
-  gdk_error_trap_push ();
+  display = gdk_screen_get_display (screen);
+  if (!GDK_IS_X11_DISPLAY (display)) {
+    g_warning ("na_tray only works on X11");
+    return NULL;
+  }
+  gdk_x11_display_error_trap_push (display);
   result = XGetWindowAttributes (xdisplay, icon_window,
                                  &window_attributes);
-  gdk_error_trap_pop_ignored ();
+  gdk_x11_display_error_trap_pop_ignored (display);
 
   if (!result) /* Window already gone */
     return NULL;
@@ -468,7 +479,7 @@ na_tray_child_get_title (NaTrayChild *child)
   utf8_string = gdk_x11_get_xatom_by_name_for_display (display, "UTF8_STRING");
   atom = gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_NAME");
 
-  gdk_error_trap_push ();
+  gdk_x11_display_error_trap_push (display);
 
   result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display),
                                child->icon_window,
@@ -478,7 +489,7 @@ na_tray_child_get_title (NaTrayChild *child)
                                &type, &format, &nitems,
                                &bytes_after, (guchar **)&val);
 
-  if (gdk_error_trap_pop () || result != Success)
+  if (gdk_x11_display_error_trap_pop (display) || result != Success)
     return NULL;
 
   if (type != utf8_string ||
@@ -593,14 +604,16 @@ _get_wmclass (Display *xdisplay,
               char   **res_class,
               char   **res_name)
 {
+  GdkDisplay *display;
   XClassHint ch;
 
   ch.res_name = NULL;
   ch.res_class = NULL;
 
-  gdk_error_trap_push ();
+  display = gdk_display_get_default ();
+  gdk_x11_display_error_trap_push (display);
   XGetClassHint (xdisplay, xwindow, &ch);
-  gdk_error_trap_pop_ignored ();
+  gdk_x11_display_error_trap_pop_ignored (display);
 
   if (res_class)
     *res_class = NULL;
