@@ -21,6 +21,11 @@
  */
 
 #include <config.h>
+
+#ifndef HAVE_X11
+#error file should only be built when HAVE_X11 is enabled
+#endif
+
 #include <string.h>
 
 #include <gtk/gtk.h>
