@@ -42,6 +42,9 @@ typedef struct {
 
 	GtkOrientation orientation;
 	int size;
+#if !defined(WNCKLET_INPROCESS) && !GTK_CHECK_VERSION (3, 23, 0)
+	gboolean needs_hints;
+#endif
 
 	GtkIconTheme* icon_theme;
 
@@ -319,7 +322,27 @@ static void applet_size_allocate(GtkWidget *widget, GtkAllocation *allocation, T
 
 	g_assert(len % 2 == 0);
 
-	mate_panel_applet_set_size_hints(MATE_PANEL_APPLET(tasklist->applet), size_hints, len, 0);
+#if !defined(WNCKLET_INPROCESS) && !GTK_CHECK_VERSION (3, 23, 0)
+	/* HACK: When loading the WnckTasklist initially, it reports size hints as though there were
+	 * no elements in the Tasklist. This causes a rendering issue when built out-of-process in
+	 * HiDPI displays. We keep a flag to skip size hinting until WnckTasklist has something to
+	 * show. */
+	if (!tasklist->needs_hints)
+	{
+		int i;
+		for (i = 0; i < len; i++)
+		{
+			if (size_hints[i])
+			{
+				tasklist->needs_hints = TRUE;
+				break;
+			}
+		}
+	}
+
+	if (tasklist->needs_hints)
+#endif
+		mate_panel_applet_set_size_hints(MATE_PANEL_APPLET(tasklist->applet), size_hints, len, 0);
 }
 
 static GdkPixbuf* icon_loader_func(const char* icon, int size, unsigned int flags, void* data)
@@ -404,6 +427,10 @@ gboolean window_list_applet_fill(MatePanelApplet* applet)
 
 	tasklist->size = mate_panel_applet_get_size(applet);
 
+#if !defined(WNCKLET_INPROCESS) && !GTK_CHECK_VERSION (3, 23, 0)
+	tasklist->needs_hints = FALSE;
+#endif
+
 	switch (mate_panel_applet_get_orient(applet))
 	{
 		case MATE_PANEL_APPLET_ORIENT_LEFT:
@@ -530,16 +557,14 @@ static void display_about_dialog(GtkAction* action, TasklistData* tasklist)
 		NULL
 	};
 
-	char copyright[] = \
-		"Copyright \xc2\xa9 2012-2018 MATE developers\n"
-		"Copyright \xc2\xa9 2011 Perberos\n"
-		"Copyright \xc2\xa9 2002 Red Hat, Inc.";
-
 	gtk_show_about_dialog(GTK_WINDOW(tasklist->applet),
 		"program-name", _("Window List"),
+		"title", _("About Window List"),
 		"authors", authors,
 		"comments", _("The Window List shows a list of all windows in a set of buttons and lets you browse them."),
-		"copyright", copyright,
+		"copyright", _("Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
+		               "Copyright \xc2\xa9 2011 Perberos\n"
+		               "Copyright \xc2\xa9 2012-2019 MATE developers"),
 		"documenters", documenters,
 		"icon-name", WINDOW_LIST_ICON,
 		"logo-icon-name", WINDOW_LIST_ICON,
