@@ -120,7 +120,8 @@ static void update_properties_for_wm(PagerData* pager)
 				gtk_widget_show(pager->workspace_names_scroll);
 			if (pager->display_workspaces_toggle)
 				gtk_widget_show(pager->display_workspaces_toggle);
-			g_object_set (pager->cell, "editable", TRUE, NULL);
+			if (pager->cell)
+				g_object_set (pager->cell, "editable", TRUE, NULL);
 			break;
 		case PAGER_WM_METACITY:
 			if (pager->workspaces_frame)
@@ -131,7 +132,8 @@ static void update_properties_for_wm(PagerData* pager)
 				gtk_widget_show(pager->workspace_names_scroll);
 			if (pager->display_workspaces_toggle)
 				gtk_widget_show(pager->display_workspaces_toggle);
-			g_object_set (pager->cell, "editable", TRUE, NULL);
+			if (pager->cell)
+				g_object_set (pager->cell, "editable", TRUE, NULL);
 			break;
 		case PAGER_WM_I3:
 			if (pager->workspaces_frame)
@@ -144,7 +146,8 @@ static void update_properties_for_wm(PagerData* pager)
 				gtk_widget_hide(pager->workspace_names_scroll);
 			if (pager->display_workspaces_toggle)
 				gtk_widget_show(pager->display_workspaces_toggle);
-			g_object_set (pager->cell, "editable", FALSE, NULL);
+			if (pager->cell)
+				g_object_set (pager->cell, "editable", FALSE, NULL);
 			break;
 		case PAGER_WM_COMPIZ:
 			if (pager->workspaces_frame)
@@ -155,7 +158,8 @@ static void update_properties_for_wm(PagerData* pager)
 				gtk_widget_hide(pager->workspace_names_scroll);
 			if (pager->display_workspaces_toggle)
 				gtk_widget_hide(pager->display_workspaces_toggle);
-			g_object_set (pager->cell, "editable", FALSE, NULL);
+			if (pager->cell)
+				g_object_set (pager->cell, "editable", FALSE, NULL);
 			break;
 		case PAGER_WM_UNKNOWN:
 			if (pager->workspaces_frame)
@@ -248,6 +252,27 @@ static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBac
                 type == PANEL_NO_BACKGROUND ? GTK_SHADOW_NONE : GTK_SHADOW_IN);
 }
 
+static void applet_style_updated (MatePanelApplet *applet, GtkStyleContext *context)
+{
+	GtkCssProvider *provider;
+	GdkRGBA color;
+
+	provider = gtk_css_provider_new ();
+
+	/* Provide a fallback color for the highlighted workspace based on the current theme */
+	gtk_style_context_lookup_color (context, "theme_selected_bg_color", &color);
+	gchar *bg_css = g_strconcat (".wnck-pager:selected {\n"
+		                     "	background-color:", gdk_rgba_to_string (&color), ";\n"
+		                     "}", NULL);
+	gtk_css_provider_load_from_data (provider, bg_css, -1, NULL);
+	g_free (bg_css);
+
+	gtk_style_context_add_provider (context,
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
+	g_object_unref (provider);
+}
+
 /* Replacement for the default scroll handler that also cares about the wrapping property.
  * Alternative: Add behaviour to libwnck (to the WnckPager widget).
  */
@@ -493,7 +518,6 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	PagerData* pager;
 	GtkActionGroup* action_group;
 	gboolean display_names;
-	GtkCssProvider *provider;
 
 	pager = g_new0(PagerData, 1);
 
@@ -546,16 +570,6 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	context = gtk_widget_get_style_context (pager->pager);
 	gtk_style_context_add_class (context, "wnck-pager");
 
-	provider = gtk_css_provider_new ();
-	gtk_css_provider_load_from_data (provider,
-                                         ".wnck-pager:selected {\n"
-                                         "background-color: #4A90D9; }",
-                                         -1, NULL);
-	gtk_style_context_add_provider (context,
-					GTK_STYLE_PROVIDER (provider),
-					GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
-	g_object_unref (provider);
-
 	g_signal_connect(G_OBJECT(pager->pager), "destroy", G_CALLBACK(destroy_pager), pager);
 
 	/* overwrite default WnckPager widget scroll-event */
@@ -567,6 +581,7 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	g_signal_connect(G_OBJECT(pager->applet), "unrealize", G_CALLBACK(applet_unrealized), pager);
 	g_signal_connect(G_OBJECT(pager->applet), "change_orient", G_CALLBACK(applet_change_orient), pager);
 	g_signal_connect(G_OBJECT(pager->applet), "change_background", G_CALLBACK(applet_change_background), pager);
+	g_signal_connect(G_OBJECT(pager->applet), "style-updated", G_CALLBACK(applet_style_updated), context);
 
 	gtk_widget_show(pager->pager);
 	gtk_widget_show(pager->applet);
@@ -615,16 +630,14 @@ static void display_about_dialog(GtkAction* action, PagerData* pager)
 		NULL
 	};
 
-	char copyright[] = \
-		"Copyright \xc2\xa9 2012-2018 MATE developers\n"
-		"Copyright \xc2\xa9 2011 Perberos\n"
-		"Copyright \xc2\xa9 2002 Red Hat, Inc.";
-
 	gtk_show_about_dialog(GTK_WINDOW(pager->applet),
 		"program-name", _("Workspace Switcher"),
+		"title", _("About Workspace Switcher"),
 		"authors", authors,
 		"comments", _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
-		"copyright", copyright,
+		"copyright", _("Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
+                               "Copyright \xc2\xa9 2011 Perberos\n"
+                               "Copyright \xc2\xa9 2012-2019 MATE developers"),
 		"documenters", documenters,
 		"icon-name", WORKSPACE_SWITCHER_ICON,
 		"logo-icon-name", WORKSPACE_SWITCHER_ICON,
