@@ -26,6 +26,7 @@
 #include <glib/gi18n.h>
 
 #include "panel-gtk.h"
+#include "panel-cleanup.h"
 
 /*
  * Originally based on code from panel-properties-dialog.c. This part of the
@@ -33,6 +34,11 @@
  * Copyright (C) 2005 Vincent Untz <vuntz@gnome.org>
  */
 
+/*There should be only one icon_settings object for the whole panel
+ *So we need a global variable here
+ */
+static GSettings *icon_settings = NULL;
+
 static void
 panel_gtk_file_chooser_preview_update (GtkFileChooser *chooser,
 				       gpointer data)
@@ -160,27 +166,54 @@ panel_file_chooser_dialog_new (const gchar          *title,
 	return result;
 }
 
+
+static void
+ensure_icon_settings (void)
+{
+	if (icon_settings != NULL)
+	return;
+
+	icon_settings = g_settings_new ("org.mate.interface");
+
+	panel_cleanup_register (panel_cleanup_unref_and_nullify,
+					&icon_settings);
+}
+
 GtkWidget *
 panel_image_menu_item_new_from_icon (const gchar *icon_name,
 				     const gchar *label_name)
 {
+	gchar *concat;
 	GtkWidget *icon;
+	GtkStyleContext *context;
 	GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+	GtkWidget *icon_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
 	if (icon_name)
 		icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	else
 		icon = gtk_image_new ();
 
-	GtkWidget *label_menu = gtk_label_new_with_mnemonic (g_strconcat (label_name, "     ", NULL));
+	concat = g_strconcat (label_name, "     ", NULL);
+	GtkWidget *label_menu = gtk_label_new_with_mnemonic (concat);
 	GtkWidget *menuitem = gtk_menu_item_new ();
  
-	gtk_container_add (GTK_CONTAINER (box), icon);
+	context = gtk_widget_get_style_context (GTK_WIDGET(icon_box));
+	gtk_style_context_add_class(context,"mate-panel-menu-icon-box");
+
+	gtk_container_add (GTK_CONTAINER (icon_box), icon);
+	gtk_container_add (GTK_CONTAINER (box), icon_box);
 	gtk_container_add (GTK_CONTAINER (box), label_menu);
  
 	gtk_container_add (GTK_CONTAINER (menuitem), box);
 	gtk_widget_show_all (menuitem);
 
+	ensure_icon_settings();
+	g_settings_bind (icon_settings, "menus-have-icons", icon, "visible",
+                         G_SETTINGS_BIND_GET);
+
+	g_free (concat);
+
 	return menuitem;
 }
 
@@ -188,23 +221,37 @@ GtkWidget *
 panel_image_menu_item_new_from_gicon (GIcon       *gicon,
 				      const gchar *label_name)
 {
+	gchar *concat;
 	GtkWidget *icon;
+	GtkStyleContext *context;
 	GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+	GtkWidget *icon_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
 	if (gicon)
 		icon = gtk_image_new_from_gicon (gicon, GTK_ICON_SIZE_MENU);
 	else
 		icon = gtk_image_new ();
 
-	GtkWidget *label_menu = gtk_label_new_with_mnemonic (g_strconcat (label_name, "     ", NULL));
+	concat = g_strconcat (label_name, "     ", NULL);
+	GtkWidget *label_menu = gtk_label_new_with_mnemonic (concat);
 	GtkWidget *menuitem = gtk_menu_item_new ();
+
+	context = gtk_widget_get_style_context (GTK_WIDGET(icon_box));
+	gtk_style_context_add_class(context,"mate-panel-menu-icon-box");
  
-	gtk_container_add (GTK_CONTAINER (box), icon);
+	gtk_container_add (GTK_CONTAINER (icon_box), icon);
+	gtk_container_add (GTK_CONTAINER (box), icon_box);
 	gtk_container_add (GTK_CONTAINER (box), label_menu);
  
 	gtk_container_add (GTK_CONTAINER (menuitem), box);
 	gtk_widget_show_all (menuitem);
 
+	ensure_icon_settings();
+	g_settings_bind (icon_settings, "menus-have-icons", icon, "visible",
+                         G_SETTINGS_BIND_GET);
+
+	g_free (concat);
+
 	return menuitem;
 }
 
@@ -214,9 +261,9 @@ panel_check_menu_item_new (GtkWidget *widget_check)
 	GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	GtkWidget *menuitem = gtk_menu_item_new ();
 	GtkWidget *label_name = gtk_bin_get_child (GTK_BIN (widget_check));
+	gchar *concat = g_strconcat (gtk_label_get_label (GTK_LABEL (label_name)), "     ", NULL);
 
-	gtk_label_set_text_with_mnemonic (GTK_LABEL (label_name),
-					  g_strconcat (gtk_label_get_label (GTK_LABEL (label_name)), "     ", NULL));
+	gtk_label_set_text_with_mnemonic (GTK_LABEL (label_name), concat);
 
 	gtk_widget_set_margin_start (widget_check, 2);
 	gtk_widget_set_margin_start (gtk_bin_get_child (GTK_BIN (widget_check)), 11);
@@ -227,5 +274,7 @@ panel_check_menu_item_new (GtkWidget *widget_check)
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label_name), menuitem);
 
+	g_free (concat);
+
 	return menuitem;
 }
