@@ -33,6 +33,7 @@
 
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-show.h>
+#include <libpanel-util/panel-gtk.h>
 
 #include "launcher.h"
 #include "panel.h"
@@ -516,13 +517,15 @@ panel_addto_prepend_directory (GSList             **parent_list,
 			       const char          *filename)
 {
 	PanelAddtoAppList *data;
+	GIcon              *gicon;
 
 	data = g_new0 (PanelAddtoAppList, 1);
+	gicon = matemenu_tree_directory_get_icon (directory);
 
 	data->item_info.type          = PANEL_ADDTO_MENU;
 	data->item_info.name          = g_strdup (matemenu_tree_directory_get_name (directory));
 	data->item_info.description   = g_strdup (matemenu_tree_directory_get_comment (directory));
-	data->item_info.icon          = g_strdup (matemenu_tree_directory_get_icon (directory));
+	data->item_info.icon          = gicon ? g_icon_to_string(gicon) : g_strdup(PANEL_ICON_UNKNOWN);
 	data->item_info.menu_filename = g_strdup (filename);
 	data->item_info.menu_path     = matemenu_tree_directory_make_path (directory, NULL);
 	data->item_info.static_data   = FALSE;
@@ -546,13 +549,18 @@ panel_addto_prepend_entry (GSList         **parent_list,
 			   const char      *filename)
 {
 	PanelAddtoAppList *data;
+	GDesktopAppInfo    *ginfo;
+	GIcon              *gicon;
+
+	ginfo = matemenu_tree_entry_get_app_info (entry);
+	gicon = g_app_info_get_icon(G_APP_INFO(ginfo));
 
 	data = g_new0 (PanelAddtoAppList, 1);
 
 	data->item_info.type          = PANEL_ADDTO_LAUNCHER;
-	data->item_info.name          = g_strdup (matemenu_tree_entry_get_display_name (entry));
-	data->item_info.description   = g_strdup (matemenu_tree_entry_get_comment (entry));
-	data->item_info.icon          = g_strdup (matemenu_tree_entry_get_icon (entry));
+	data->item_info.name          = g_strdup (g_app_info_get_display_name(G_APP_INFO(ginfo)));
+	data->item_info.description   = g_strdup (g_app_info_get_description(G_APP_INFO(ginfo)));
+	data->item_info.icon          = gicon ? g_icon_to_string(gicon) : g_strdup(PANEL_ICON_UNKNOWN);
 	data->item_info.launcher_path = g_strdup (matemenu_tree_entry_get_desktop_file_path (entry));
 	data->item_info.static_data   = FALSE;
 
@@ -564,28 +572,28 @@ panel_addto_prepend_alias (GSList         **parent_list,
 			   MateMenuTreeAlias  *alias,
 			   const char      *filename)
 {
-	MateMenuTreeItem *aliased_item;
+	gpointer item;
 
-	aliased_item = matemenu_tree_alias_get_item (alias);
-
-	switch (matemenu_tree_item_get_type (aliased_item)) {
+	switch (matemenu_tree_alias_get_aliased_item_type (alias)) {
 	case MATEMENU_TREE_ITEM_DIRECTORY:
+		item = matemenu_tree_alias_get_directory(alias);
 		panel_addto_prepend_directory (parent_list,
-					       MATEMENU_TREE_DIRECTORY (aliased_item),
-					       filename);
+				item,
+				filename);
+		matemenu_tree_item_unref (item);
 		break;
 
 	case MATEMENU_TREE_ITEM_ENTRY:
+		item = matemenu_tree_alias_get_aliased_entry(alias);
 		panel_addto_prepend_entry (parent_list,
-					   MATEMENU_TREE_ENTRY (aliased_item),
-					   filename);
+				item,
+				filename);
+		matemenu_tree_item_unref (item);
 		break;
 
 	default:
 		break;
 	}
-
-	matemenu_tree_item_unref (aliased_item);
 }
 
 static void
@@ -593,33 +601,34 @@ panel_addto_make_application_list (GSList             **parent_list,
 				   MateMenuTreeDirectory  *directory,
 				   const char          *filename)
 {
-	GSList *items;
-	GSList *l;
-
-	items = matemenu_tree_directory_get_contents (directory);
-
-	for (l = items; l; l = l->next) {
-		switch (matemenu_tree_item_get_type (l->data)) {
+	MateMenuTreeIter *iter;
+	iter = matemenu_tree_directory_iter (directory);
+	MateMenuTreeItemType type;
+	while ((type = matemenu_tree_iter_next (iter)) != MATEMENU_TREE_ITEM_INVALID) {
+		gpointer item;
+		switch (type) {
 		case MATEMENU_TREE_ITEM_DIRECTORY:
-			panel_addto_prepend_directory (parent_list, l->data, filename);
+			item = matemenu_tree_iter_get_directory(iter);
+			panel_addto_prepend_directory (parent_list, item, filename);
+			matemenu_tree_item_unref (item);
 			break;
 
 		case MATEMENU_TREE_ITEM_ENTRY:
-			panel_addto_prepend_entry (parent_list, l->data, filename);
+			item = matemenu_tree_iter_get_entry (iter);
+			panel_addto_prepend_entry (parent_list, item, filename);
+			matemenu_tree_item_unref (item);
 			break;
 
 		case MATEMENU_TREE_ITEM_ALIAS:
-			panel_addto_prepend_alias (parent_list, l->data, filename);
+			item = matemenu_tree_iter_get_alias(iter);
+			panel_addto_prepend_alias (parent_list, item, filename);
+			matemenu_tree_item_unref (item);
 			break;
-
 		default:
 			break;
 		}
-
-		matemenu_tree_item_unref (l->data);
 	}
-
-	g_slist_free (items);
+	matemenu_tree_iter_unref (iter);
 
 	*parent_list = g_slist_reverse (*parent_list);
 }
@@ -661,16 +670,21 @@ static void panel_addto_make_application_model(PanelAddtoDialog* dialog)
 	GtkTreeStore* store;
 	MateMenuTree* tree;
 	MateMenuTreeDirectory* root;
+	GError *error = NULL;
 
 	if (dialog->filter_application_model != NULL)
 		return;
 
 	store = gtk_tree_store_new(NUMBER_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
 
-	tree = matemenu_tree_lookup("mate-applications.menu", MATEMENU_TREE_FLAGS_NONE);
-	matemenu_tree_set_sort_key(tree, MATEMENU_TREE_SORT_DISPLAY_NAME);
+	tree = matemenu_tree_new ("mate-applications.menu", MATEMENU_TREE_FLAGS_SORT_DISPLAY_NAME);
+	if (! matemenu_tree_load_sync (tree, &error)) {
+		g_warning("Applications menu tree loading got error:%s\n", error->message);
+		g_error_free(error);
+		g_clear_object(&tree);
+	}
 
-	if ((root = matemenu_tree_get_root_directory (tree)))
+	if ((root = matemenu_tree_get_root_directory (tree)) != NULL )
 	{
 		panel_addto_make_application_list(&dialog->application_list, root, "mate-applications.menu");
 		panel_addto_populate_application_model(store, NULL, dialog->application_list);
@@ -678,10 +692,14 @@ static void panel_addto_make_application_model(PanelAddtoDialog* dialog)
 		matemenu_tree_item_unref(root);
 	}
 
-	matemenu_tree_unref(tree);
+	g_clear_object(&tree);
 
-	tree = matemenu_tree_lookup("mate-settings.menu", MATEMENU_TREE_FLAGS_NONE);
-	matemenu_tree_set_sort_key(tree, MATEMENU_TREE_SORT_DISPLAY_NAME);
+	tree = matemenu_tree_new ("mate-settings.menu", MATEMENU_TREE_FLAGS_SORT_DISPLAY_NAME);
+	if (! matemenu_tree_load_sync (tree, &error)) {
+		g_warning("Settings menu tree loading got error:%s\n", error->message);
+		g_error_free(error);
+		g_clear_object(&tree);
+	}
 
 	if ((root = matemenu_tree_get_root_directory(tree)))
 	{
@@ -696,7 +714,7 @@ static void panel_addto_make_application_model(PanelAddtoDialog* dialog)
 		matemenu_tree_item_unref(root);
 	}
 
-	matemenu_tree_unref(tree);
+	g_object_unref(tree);
 
 	dialog->application_model = GTK_TREE_MODEL(store);
 	dialog->filter_application_model = gtk_tree_model_filter_new(GTK_TREE_MODEL(dialog->application_model), NULL);
@@ -969,9 +987,7 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 		g_object_unref (dialog->application_model);
 	dialog->application_model = NULL;
 
-	if (dialog->menu_tree)
-		matemenu_tree_unref (dialog->menu_tree);
-	dialog->menu_tree = NULL;
+	g_clear_object (&dialog->menu_tree);
 
 	g_free (dialog);
 }
@@ -1220,17 +1236,22 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 			  dialog);
 
 	dialog->addto_dialog = gtk_dialog_new ();
-	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       "gtk-help", GTK_RESPONSE_HELP);
-	dialog->back_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-						     "gtk-go-back",
-						     PANEL_ADDTO_RESPONSE_BACK);
-	dialog->add_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-						     "gtk-add",
-						     PANEL_ADDTO_RESPONSE_ADD);
-	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       "gtk-close",
-			       GTK_RESPONSE_CLOSE);
+
+	panel_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+				 _("_Help"), "help-browser", GTK_RESPONSE_HELP);
+
+	dialog->back_button = panel_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+						       _("_Back"), "go-previous",
+						       PANEL_ADDTO_RESPONSE_BACK);
+
+	dialog->add_button = panel_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+						      _("_Add"), "list-add",
+						      PANEL_ADDTO_RESPONSE_ADD);
+
+	panel_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+				 _("_Close"), "window-close",
+				 GTK_RESPONSE_CLOSE);
+
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->addto_dialog),
