@@ -33,6 +33,8 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
+#include <libpanel-util/panel-gtk.h>
+
 #include "panel-applets-manager.h"
 #include "panel-profile.h"
 #include "panel.h"
@@ -123,11 +125,7 @@ mate_panel_applet_frame_draw (GtkWidget *widget,
 			       "background-image", &bg_pattern,
 			       NULL);
 
-#if GTK_CHECK_VERSION (3, 18, 0)
 	background = &frame->priv->panel->toplevel->background;
-#else
-	background = &frame->priv->panel->background;
-#endif
 	if (bg_pattern && (background->type == PANEL_BACK_IMAGE ||
 	    (background->type == PANEL_BACK_COLOR && background->has_alpha))) {
 		cairo_matrix_t ptm;
@@ -172,11 +170,8 @@ mate_panel_applet_frame_update_background_size (MatePanelAppletFrame *frame,
 	    old_allocation->width  == new_allocation->width &&
 	    old_allocation->height == new_allocation->height)
 		return;
-#if GTK_CHECK_VERSION (3, 18, 0)
+
 	background = &frame->priv->panel->toplevel->background;
-#else
-	background = &frame->priv->panel->background;
-#endif
 	if (background->type == PANEL_BACK_NONE ||
 	   (background->type == PANEL_BACK_COLOR && !background->has_alpha))
 		return;
@@ -385,12 +380,7 @@ mate_panel_applet_frame_button_changed (GtkWidget      *widget,
 	MatePanelAppletFrame *frame;
 	gboolean              handled = FALSE;
 	GdkDisplay *display;
-#if GTK_CHECK_VERSION (3, 20, 0)
 	GdkSeat *seat;
-#else
-	GdkDevice *pointer;
-	GdkDeviceManager *device_manager;
-#endif
 
 	frame = MATE_PANEL_APPLET_FRAME (widget);
 
@@ -419,16 +409,9 @@ mate_panel_applet_frame_button_changed (GtkWidget      *widget,
 	case 3:
 		if (event->type == GDK_BUTTON_PRESS ||
 		    event->type == GDK_2BUTTON_PRESS) {
-#if GTK_CHECK_VERSION(3, 20, 0)
 			display = gtk_widget_get_display (widget);
 			seat = gdk_display_get_default_seat (display);
 			gdk_seat_ungrab (seat);
-#else
-			display = gtk_widget_get_display (widget);
-			device_manager = gdk_display_get_device_manager (display);
-			pointer = gdk_device_manager_get_client_pointer (device_manager);
-			gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
-#endif
 
 			MATE_PANEL_APPLET_FRAME_GET_CLASS (frame)->popup_menu (frame,
 									  event->button,
@@ -550,11 +533,7 @@ mate_panel_applet_frame_change_background (MatePanelAppletFrame    *frame,
 
 	if (frame->priv->has_handle) {
 		PanelBackground *background;
-#if GTK_CHECK_VERSION (3, 18, 0)
 		background = &PANEL_WIDGET (parent)->toplevel->background;
-#else
-		background = &PANEL_WIDGET (parent)->background;
-#endif
 		panel_background_apply_css (background, GTK_WIDGET (frame));
 	}
 
@@ -645,11 +624,7 @@ _mate_panel_applet_frame_update_flags (MatePanelAppletFrame *frame,
 		 * it */
 		PanelBackground *background;
 
-#if GTK_CHECK_VERSION (3, 18, 0)
 		background = &frame->priv->panel->toplevel->background;
-#else
-		background = &frame->priv->panel->background;
-#endif
 		mate_panel_applet_frame_change_background (frame, background->type);
 	}
 }
@@ -705,11 +680,8 @@ _mate_panel_applet_frame_get_background_string (MatePanelAppletFrame    *frame,
 			break;
 		}
 	}
-#if GTK_CHECK_VERSION (3, 18, 0)
+
 	return panel_background_make_string (&panel->toplevel->background, x, y);
-#else
-	return panel_background_make_string (&panel->background, x, y);
-#endif
 }
 
 static void
@@ -827,6 +799,10 @@ _mate_panel_applet_frame_applet_broken (MatePanelAppletFrame *frame)
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
 
 	gtk_widget_show (dialog);
+
+	gtk_window_present_with_time (GTK_WINDOW (dialog),
+				      gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (dialog))));
+
 	g_free (dialog_txt);
 }
 
@@ -974,17 +950,18 @@ mate_panel_applet_frame_loading_failed (const char  *iid,
 	g_free (problem_txt);
 
 	if (locked_down) {
-		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-					GTK_STOCK_OK, LOADING_FAILED_RESPONSE_DONT_DELETE,
-					NULL);
+		panel_dialog_add_button (GTK_DIALOG (dialog),
+					 _("_OK"), "gtk-ok", LOADING_FAILED_RESPONSE_DONT_DELETE);
 	} else {
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 					_("Do you want to delete the applet "
 					  "from your configuration?"));
-		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-					PANEL_STOCK_DONT_DELETE, LOADING_FAILED_RESPONSE_DONT_DELETE,
-					GTK_STOCK_DELETE, LOADING_FAILED_RESPONSE_DELETE,
-					NULL);
+
+		gtk_dialog_add_button (GTK_DIALOG (dialog),
+				       PANEL_STOCK_DONT_DELETE, LOADING_FAILED_RESPONSE_DONT_DELETE);
+
+		panel_dialog_add_button (GTK_DIALOG (dialog),
+					 _("_Delete"), "edit-delete", LOADING_FAILED_RESPONSE_DELETE);
 	}
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
