@@ -24,6 +24,10 @@
 
 #include <config.h>
 
+#ifndef HAVE_X11
+#error file should only be built when HAVE_X11 is enabled
+#endif
+
 #include "panel-force-quit.h"
 
 #include <glib/gi18n.h>
@@ -122,9 +126,10 @@ remove_popup (GtkWidget *popup)
 }
 
 static gboolean
-wm_state_set (Display *display,
+wm_state_set (Display *xdisplay,
 	      Window   window)
 {
+	GdkDisplay *display;
 	gulong  nitems;
 	gulong  bytes_after;
 	gulong *prop;
@@ -132,13 +137,14 @@ wm_state_set (Display *display,
 	int     ret_format;
 	int     result;
 
-	gdk_error_trap_push ();
-	result = XGetWindowProperty (display, window, wm_state_atom,
+	display = gdk_display_get_default ();
+	gdk_x11_display_error_trap_push (display);
+	result = XGetWindowProperty (xdisplay, window, wm_state_atom,
 				     0, G_MAXLONG, False, wm_state_atom,
 				     &ret_type, &ret_format, &nitems,
 				     &bytes_after, (gpointer) &prop);
 
-	if (gdk_error_trap_pop ())
+	if (gdk_x11_display_error_trap_pop (display))
 		return FALSE;
 
 	if (result != Success)
@@ -153,9 +159,10 @@ wm_state_set (Display *display,
 }
 
 static Window 
-find_managed_window (Display *display,
+find_managed_window (Display *xdisplay,
 		     Window   window)
 {
+	GdkDisplay *display;
 	Window  root;
 	Window  parent;
 	Window *kids = NULL;
@@ -163,23 +170,24 @@ find_managed_window (Display *display,
 	guint   nkids;
 	int     i, result;
 
-	if (wm_state_set (display, window))
+	if (wm_state_set (xdisplay, window))
 		return window;
 
-	gdk_error_trap_push ();
-	result = XQueryTree (display, window, &root, &parent, &kids, &nkids);
-	if (gdk_error_trap_pop () || !result)
+	display = gdk_display_get_default ();
+	gdk_x11_display_error_trap_push (display);
+	result = XQueryTree (xdisplay, window, &root, &parent, &kids, &nkids);
+	if (gdk_x11_display_error_trap_pop (display) || !result)
 		return None;
 
 	retval = None;
 
 	for (i = 0; i < nkids; i++) {
-		if (wm_state_set (display, kids [i])) {
+		if (wm_state_set (xdisplay, kids [i])) {
 			retval = kids [i];
 			break;
 		}
 
-		retval = find_managed_window (display, kids [i]);
+		retval = find_managed_window (xdisplay, kids [i]);
 		if (retval != None)
 			break;
 	}
@@ -196,15 +204,17 @@ kill_window_response (GtkDialog *dialog,
 		      gpointer   user_data)
 {
 	if (response_id == GTK_RESPONSE_ACCEPT) {
-		Display *display;
+		GdkDisplay *display;
+		Display *xdisplay;
 		Window window = (Window) user_data;
 
-		display = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (GTK_WIDGET (dialog)));
+		display = gtk_widget_get_display (GTK_WIDGET (dialog));
+		xdisplay = GDK_DISPLAY_XDISPLAY (display);
 
-		gdk_error_trap_push ();
-		XKillClient (display, window);
-		gdk_flush ();
-		gdk_error_trap_pop_ignored ();
+		gdk_x11_display_error_trap_push (display);
+		XKillClient (xdisplay, window);
+		gdk_display_flush (display);
+		gdk_x11_display_error_trap_pop_ignored (display);
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -309,19 +319,28 @@ popup_filter (GdkXEvent *gdk_xevent,
 	return GDK_FILTER_CONTINUE;
 }
 
+static void
+prepare_root_window (GdkSeat   *seat,
+                     GdkWindow *window,
+                     gpointer   user_data)
+{
+	gdk_window_show_unraised (window);
+}
+
 void
 panel_force_quit (GdkScreen *screen,
 		  guint      time)
 {
 	GdkGrabStatus  status;
 	GdkCursor     *cross;
+	GdkSeatCapabilities caps;
 	GtkWidget     *popup;
 	GdkWindow     *root;
 	GdkDisplay    *display;
-	GdkDevice     *pointer;
-	GdkDevice     *keyboard;
 	GdkSeat       *seat;
 
+	g_return_if_fail (GDK_IS_X11_DISPLAY (gdk_screen_get_display (screen)));
+
 	popup = display_popup_window (screen);
 
 	root = gdk_screen_get_root_window (screen);
@@ -329,28 +348,23 @@ panel_force_quit (GdkScreen *screen,
 	gdk_window_add_filter (root, (GdkFilterFunc) popup_filter, popup);
 	cross = gdk_cursor_new_for_display (gdk_display_get_default (),
 	                                    GDK_CROSS);
+	caps = GDK_SEAT_CAPABILITY_POINTER | GDK_SEAT_CAPABILITY_KEYBOARD;
 	display = gdk_window_get_display (root);
 	seat = gdk_display_get_default_seat (display);
-	pointer = gdk_seat_get_pointer (seat);
-	keyboard = gdk_device_get_associated_device (pointer);
 
-	status = gdk_device_grab (pointer, root,
-				  GDK_OWNERSHIP_NONE, FALSE,
-				  GDK_BUTTON_PRESS_MASK,
-				  cross, time);
+	status = gdk_seat_grab (seat, root,
+	                        caps, FALSE,
+	                        cross, NULL,
+	                        prepare_root_window,
+	                        NULL);
 
 	g_object_unref (cross);
 
-	status = gdk_device_grab (keyboard, root,
-				  GDK_OWNERSHIP_NONE, FALSE,
-				  GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
-				  NULL, time);
-
 	if (status != GDK_GRAB_SUCCESS) {
 		g_warning ("Pointer grab failed\n");
 		remove_popup (popup);
 		return;
 	}
 
-	gdk_flush ();
+	gdk_display_flush (display);
 }
